/******************************************************************************
 libprozilla - a download accelerator library
 Copyright (C) 2001 Kalum Somaratna

 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
******************************************************************************/

/* A test program. */

/* $Id: test.c,v 1.26 2001/05/30 15:01:40 kalum Exp $ */


#include "common.h"
#include "prozilla.h"
#include "connection.h"
#include "misc.h"
#include "connect.h"
#include "ftp.h"
#include "http.h"
#include "download.h"


/******************************************************************************
 The main function.
******************************************************************************/
int main(int argc, char **argv)
{
  uerr_t err;
  urlinfo url_data;
  connection_t conn;
  pthread_mutex_t mutex;
  proxy_info http_proxy;
  proxy_info ftp_proxy;
  download_t *dl;
  int num_connections;


  proz_init(argc, argv);
  libprozrtinfo.conn_timeout.tv_sec = 100;

  memset(&url_data, 0, sizeof(url_data));
  memset(&http_proxy, 0, sizeof(proxy_info));
  memset(&ftp_proxy, 0, sizeof(proxy_info));

  if (argc != 2)
    err =
	proz_parse_url("ftp://localhost/pub/gcc-2.95.2.tar.bz2", &url_data,
		       0);
  else
    err = proz_parse_url(argv[1], &url_data, 0);


  /* Experiment with the proxy a bit. */
  proz_parse_url("localhost:8080", &http_proxy.proxy_url, 0);

  proz_set_http_proxy(&http_proxy);
  proz_connection_init(&conn, &mutex);

  memcpy(&conn.u, &url_data, sizeof(url_data));

  /* conn.use_http_proxy = TRUE; */

  if (url_data.proto == URLFTP)
  {
    err = proz_ftp_get_url_info(&conn);
    if (err != FTPOK)
    {
      printf("Bad Error.\n");
      exit(EXIT_FAILURE);
    }
  } else
  {
    err = proz_get_url_info_loop(&conn);
    if (err != HOK)
    {
      printf("Bad Error.\n");
      exit(EXIT_FAILURE);
    }
  }

  if (conn.file_type == REGULAR_FILE)
    printf("File Size: %ld.\n", conn.main_file_size);
  else
    printf("Is not a file.\n");

  /*  proz_connection_cleanup(&conn); */

  dl = proz_download_init(&url_data);

  num_connections =
      proz_download_setup_connections_no_ftpsearch(dl, &conn, 1);
  proz_download_load_resume_info(dl);
  proz_download_start_downloads(dl, FALSE);

  while (1);
  /*
     init_response(&conn);
     ftp_connect_to_server(&conn, url_data.host, url_data.port);
     done_with_response(&conn);

     init_response(&conn);
     ftp_login(&conn, "anonymous", "sd");
     done_with_response(&conn);
   */

  proz_shutdown();

  exit(EXIT_SUCCESS);
}
